@@ -73,7 +73,8 @@ namespace pl::core {
             Catch,
             Import,
             As,
-            Is
+            Is,
+            From
         };
 
         enum class Operator {
@@ -133,7 +134,7 @@ namespace pl::core {
             Double         = 0x82,
             String         = 0x15,
             Auto           = 0x16,
-            CustomType     = 0x00,
+            CustomType     = 0x0F,
             Padding        = 0x1F,
 
             Unsigned      = 0xFF00,
@@ -179,10 +180,11 @@ namespace pl::core {
                 Typedef,
                 Function,
                 UDT,
+                View,
                 FunctionVariable,
                 FunctionParameter,
-                PatternLocalVariable,
-                PatternPlacedVariable,
+                LocalVariable,
+                CalculatedPointer,
                 TemplateArgument,
                 PatternVariable,
                 GlobalVariable,
