@@ -25,7 +25,7 @@ namespace pl::core::ast {
         this->m_constant    = other.m_constant;
     }
 
-    [[nodiscard]] std::vector<std::shared_ptr<ptrn::Pattern>> ASTNodeVariableDecl::createPatterns(Evaluator *evaluator) const {
+    void ASTNodeVariableDecl::createPatterns(Evaluator *evaluator, std::vector<std::shared_ptr<ptrn::Pattern>> &resultPatterns) const {
         [[maybe_unused]] auto context = evaluator->updateRuntime(this);
 
         auto startOffset = evaluator->getBitwiseReadOffset();
@@ -40,7 +40,7 @@ namespace pl::core::ast {
             if (id == nullptr)
                 err::E0002.throwError("Cannot use void expression as section identifier.", {}, this->getLocation());
 
-            evaluator->pushSectionId(id->getValue().toUnsigned());
+            evaluator->pushSectionId((u64)id->getValue().toUnsigned());
         } else {
             scopeGuard.release();
         }
@@ -54,7 +54,7 @@ namespace pl::core::ast {
             evaluator->setReadOffset(std::visit(wolv::util::overloaded {
                 [this](const std::string &) -> u64 { err::E0005.throwError("Cannot use string as placement offset.", "Try using a integral value instead.", this->getLocation()); },
                 [this](const std::shared_ptr<ptrn::Pattern> &) -> u64 { err::E0005.throwError("Cannot use string as placement offset.", "Try using a integral value instead.", this->getLocation()); },
-                [](auto &&offset) -> u64 { return offset; } },
+                [](auto &&offset) -> u64 { return u64(offset); } },
             offset->getValue()));
 
             if (evaluator->getReadOffset() < evaluator->getDataBaseAddress() || evaluator->getReadOffset() > evaluator->getDataBaseAddress() + evaluator->getDataSize())
@@ -64,33 +64,36 @@ namespace pl::core::ast {
         if (evaluator->getSectionId() == ptrn::Pattern::PatternLocalSectionId || evaluator->getSectionId() == ptrn::Pattern::HeapSectionId) {
             evaluator->setBitwiseReadOffset(startOffset);
             this->execute(evaluator);
-            return { };
         } else {
-            auto patterns = this->m_type->createPatterns(evaluator);
+            std::vector<std::shared_ptr<ptrn::Pattern>> patterns;
+            ON_SCOPE_EXIT {
+                if (!patterns.empty()) {
+                    auto &pattern = patterns.front();
+                    if (this->m_placementOffset != nullptr && dynamic_cast<ptrn::PatternString*>(pattern.get()) != nullptr)
+                        err::E0005.throwError(fmt::format("Variables of type 'str' cannot be placed in memory.", this->m_name), { }, this->getLocation());
+
+                    pattern->setVariableName(this->m_name);
+
+                    if (this->m_placementSection != nullptr)
+                        pattern->setSection(evaluator->getSectionId());
+
+                    applyVariableAttributes(evaluator, this, pattern);
+
+                    if (this->m_placementOffset != nullptr && !evaluator->isGlobalScope()) {
+                        evaluator->setBitwiseReadOffset(startOffset);
+                    }
+
+                    if (this->m_placementSection != nullptr && !evaluator->isGlobalScope()) {
+                        evaluator->addPattern(std::move(pattern));
+                    } else {
+                        resultPatterns = hlp::moveToVector<std::shared_ptr<ptrn::Pattern>>(std::move(pattern));
+                    }
+                }
+            };
+
+            this->m_type->createPatterns(evaluator, patterns);
             if (patterns.empty())
                 err::E0005.throwError("'auto' can only be used with parameters.", { }, this->getLocation());
-
-            auto &pattern = patterns.front();
-            if (this->m_placementOffset != nullptr && dynamic_cast<ptrn::PatternString*>(pattern.get()) != nullptr)
-                err::E0005.throwError(fmt::format("Variables of type 'str' cannot be placed in memory.", this->m_name), { }, this->getLocation());
-
-            pattern->setVariableName(this->m_name);
-
-            if (this->m_placementSection != nullptr)
-                pattern->setSection(evaluator->getSectionId());
-
-            applyVariableAttributes(evaluator, this, pattern);
-
-            if (this->m_placementOffset != nullptr && !evaluator->isGlobalScope()) {
-                evaluator->setBitwiseReadOffset(startOffset);
-            }
-
-            if (this->m_placementSection != nullptr && !evaluator->isGlobalScope()) {
-                evaluator->addPattern(std::move(pattern));
-                return {};
-            }
-
-            return hlp::moveToVector<std::shared_ptr<ptrn::Pattern>>(std::move(pattern));
         }
     }
 
@@ -112,7 +115,7 @@ namespace pl::core::ast {
                 err::E0002.throwError("Cannot use void expression as section identifier.", {}, this->getLocation());
 
             auto value = sectionLiteral->getValue();
-            section = value.toUnsigned();
+            section = u64(value.toUnsigned());
         }
 
         return section;
@@ -129,7 +132,7 @@ namespace pl::core::ast {
         std::vector<std::shared_ptr<ptrn::Pattern>> initValues;
         if (this->m_placementOffset == nullptr) {
             evaluator->pushSectionId(ptrn::Pattern::InstantiationSectionId);
-            initValues = this->getType()->createPatterns(evaluator);
+            this->getType()->createPatterns(evaluator, initValues);
             evaluator->popSectionId();
         } else {
             evaluator->pushSectionId(this->m_placementSection == nullptr ? ptrn::Pattern::MainSectionId : evaluator->getSectionId());
@@ -137,8 +140,8 @@ namespace pl::core::ast {
             auto currOffset = evaluator->getBitwiseReadOffset();
             ON_SCOPE_EXIT { evaluator->setBitwiseReadOffset(currOffset); };
 
-            evaluator->setReadOffset(this->evaluatePlacementOffset(evaluator));
-            initValues = this->getType()->createPatterns(evaluator);
+            evaluator->setReadOffset(u64(this->evaluatePlacementOffset(evaluator)));
+            this->getType()->createPatterns(evaluator, initValues);
             evaluator->popSectionId();
         }
 
@@ -166,7 +169,7 @@ namespace pl::core::ast {
         if (this->m_placementOffset != nullptr) {
             auto section = this->evaluatePlacementSection(evaluator);
             auto offset = this->evaluatePlacementOffset(evaluator);
-            evaluator->setVariableAddress(this->getName(), offset, section);
+            evaluator->setVariableAddress(this->getName(), u64(offset), section);
         }
 
         return std::nullopt;
