@@ -1,5 +1,7 @@
 #pragma once
 
+#include <hex/helpers/keys.hpp>
+
 #if defined(OS_MACOS)
 
     struct GLFWwindow;
@@ -17,8 +19,15 @@
         void enumerateFontsMacos();
     
         void macosHandleTitlebarDoubleClickGesture(GLFWwindow *window);
+        void macosSetWindowMovable(GLFWwindow *window, bool movable);
         bool macosIsWindowBeingResizedByUser(GLFWwindow *window);
         void macosMarkContentEdited(GLFWwindow *window, bool edited = true);
+
+        void macosGetKey(Keys key, int *output);
+
+        bool macosIsMainInstance();
+        void macosSendMessageToMainInstance(const unsigned char *data, size_t size);
+        void macosInstallEventListener();
     }
 
 #endif
